@@ -30,6 +30,10 @@
 #' \code{mvQuad::createNIGrid(length(theta),'GHe',k,'product')}. **Note**: the \code{mvQuad}
 #' functions used within \code{aghq} operate on grids in memory, so your \code{basegrid}
 #' object will be changed after you run \code{aghq}.
+#' @param transformation Optional. Do the quadrature for parameter \code{theta}, but
+#' return summaries and plots for parameter \code{g(theta)}.
+#' \code{transformation} is either: a) an \code{aghqtrans} object returned by \code{aghq::make_transformation},
+#' or b) a list that will be passed to that function internally. See \code{?aghq::make_transformation} for details.
 #'
 #' @return An object of class \code{aghq} which is a list containing elements:
 #' \itemize{
@@ -93,9 +97,13 @@
 #'
 #' @export
 #'
-aghq <- function(ff,k,startingvalue,optresults = NULL,basegrid = NULL,control = default_control(),...) {
+aghq <- function(ff,k,startingvalue,transformation = default_transformation(),optresults = NULL,basegrid = NULL,control = default_control(),...) {
 
   validate_control(control)
+  validate_transformation(transformation)
+  transformation <- make_transformation(transformation)
+  # ff <- make_moment_function(ff)
+  # validate_moment(ff)
 
   # If they provided a basegrid, get the k from that. If they also provided a k, compare them and issue a warning
   if (!is.null(basegrid)) {
@@ -118,29 +126,36 @@ aghq <- function(ff,k,startingvalue,optresults = NULL,basegrid = NULL,control =
 
   if (control$onlynormconst) return(normalized_posterior$lognormconst)
 
+  out <- list(
+    normalized_posterior = normalized_posterior,
+    # marginals = marginals,
+    optresults = optresults,
+    control = control,
+    transformation = transformation
+  )
+  class(out) <- "aghq"
   # Marginals
   d <- length(startingvalue)
   marginals <- vector(mode = "list",length = d)
-  for (j in 1:d) marginals[[j]] <- marginal_posterior(optresults,k,j,basegrid,control$ndConstruction)
-
-  out <- list(
-    normalized_posterior = normalized_posterior,
-    marginals = marginals,
-    optresults = optresults,
-    control = control
-  )
-  class(out) <- "aghq"
+  if (control$method_summaries[1] == 'correct') {
+    for (j in 1:d) marginals[[j]] <- marginal_posterior.aghq(out,j,method = 'correct')
+  } else {
+    for (j in 1:d) marginals[[j]] <- marginal_posterior.aghq(out,j,method = 'reuse')
+  }
+  out$marginals <- marginals
   out
 }
 
 #' Summary statistics computed using AGHQ
 #'
 #' The \code{summary.aghq} method computes means, standard deviations, and
-#' quantiles and the associated print method
+#' quantiles of the transformed parameter.
+#' The associated print method
 #' prints these along with diagnostic and other information about
 #' the quadrature.
 #'
-#' @param object The return value from \code{aghq::aghq}.
+#' @param object The return value from \code{aghq::aghq}. Summaries are computed for
+#' \code{object$transformation$fromtheta(theta)}.
 #' @param ... not used.
 #'
 #' @return A list of class \code{aghqsummary}, which has a print method. Elements:
@@ -148,11 +163,11 @@ aghq <- function(ff,k,startingvalue,optresults = NULL,basegrid = NULL,control =
 #' \item{mode: }{the mode of the log posterior}
 #' \item{hessian: }{the hessian of the log posterior at the mode}
 #' \item{covariance: }{the inverse of the hessian of the log posterior at the mode}
-#' \item{cholesky: }{the upper cholesky trinagle of the hessian of the log posterior at the mode}
+#' \item{cholesky: }{the upper Cholesky triangle of the hessian of the log posterior at the mode}
 #' \item{quadpoints: }{the number of quadrature points used in each dimension}
 #' \item{dim: }{the dimension of the parameter space}
 #' \item{summarytable: }{a table containing the mean, median, mode, standard deviation
-#' and quantiles of each parameter, computed according to the posterior normalized
+#' and quantiles of each transformed parameter, computed according to the posterior normalized
 #' using AGHQ}
 #' }
 #'
@@ -197,15 +212,23 @@ aghq <- function(ff,k,startingvalue,optresults = NULL,basegrid = NULL,control =
 #' @export
 #'
 summary.aghq <- function(object,...) {
-  d <- length(object$optresults$mode)
+  d <- get_param_dim(object)
 
   thetanames <- colnames(object$normalized_posterior$nodesandweights)[1:d]
 
+  do_correct <- object$control$method_summaries[1] == 'correct'
+
   # Moments
-  themeans <- compute_moment(object$normalized_posterior,function(x) x)
-  thesds <- numeric(d)
-  for (j in 1:d) thesds[j] <- sqrt(compute_moment(object$normalized_posterior,function(x) (x - themeans[j])^2)[j])
-  names(thesds) <- names(themeans) <- thetanames
+  if (do_correct) {
+    themeans <- compute_moment(object,nn = 1,type = 'raw',method = 'correct')
+    thesds <- sqrt(compute_moment(object,nn = 2,type = 'central',method = 'correct'))
+  } else {
+    themeans <- compute_moment(object$normalized_posterior,object$transformation$fromtheta)
+    thesds <- numeric(d)
+    for (j in 1:d) thesds[j] <- sqrt(compute_moment(object$normalized_posterior,function(x) (object$transformation$fromtheta(x) - themeans[j])^2)[j])
+    names(thesds) <- names(themeans) <- thetanames
+  }
+
 
   themoments <- cbind(themeans,thesds)
   colnames(themoments) <- c('mean','sd')
@@ -213,13 +236,17 @@ summary.aghq <- function(object,...) {
 
   # Quantiles
   thequants <- vector(mode = 'list',length = d)
-  for (j in 1:d) thequants[[j]] <- compute_quantiles(object$marginals[[j]],c(.025,.5,.975),interpolation = object$control$interpolation)
+  for (j in 1:d) thequants[[j]] <- compute_quantiles(object$marginals[[j]],q = c(.025,.5,.975),transformation = object$transformation,interpolation = object$control$interpolation)
+
   names(thequants) <- thetanames
   thequants <- t(as.data.frame(thequants))
   colnames(thequants)[2] <- 'median'
 
-  thesummary <- cbind(themoments,thequants,data.frame(mode = object$optresults$mode))
-  thesummary <- thesummary[ ,c('mean','median','mode','sd','2.5%','97.5%')]
+  # thesummary <- cbind(themoments,thequants,data.frame(mode = object$optresults$mode))
+  thesummary <- cbind(themoments,thequants)
+
+  # thesummary <- thesummary[ ,c('mean','median','mode','sd','2.5%','97.5%')]
+  thesummary <- thesummary[ ,c('mean','sd','2.5%','median','97.5%')]
 
 
   out <- list()
@@ -228,7 +255,7 @@ summary.aghq <- function(object,...) {
   out$hessian <- object$optresults$hessian
   out$lognormconst <- object$normalized_posterior$lognormconst
   out$covariance <- solve(out$hessian)
-  out$cholesky <- chol(out$covariance)
+  # out$cholesky <- chol(out$covariance)
   out$quadpoints <- as.numeric(object$normalized_posterior$grid$level)
   out$dim <- length(out$quadpoints)
   out$summarytable <- thesummary
@@ -351,16 +378,17 @@ print.aghqsummary <- function(x,...) {
   # cat("...and its Cholesky is:\n")
   # print(as.matrix(x$cholesky))
   cat("\n")
-  cat("Here are some moments and quantiles for theta:\n\n")
+  cat("Here are some moments and quantiles for the transformed parameter:\n\n")
   print(x$summarytable)
   cat("\n")
 }
 
 #' Plot method for AGHQ objects
 #'
-#' Plot the marginal pdf and cdf from an \code{aghq} object.
+#' Plot the marginal pdf and cdf of the transformed parameter from an \code{aghq} object.
 #'
-#' @param x The return value of \code{aghq::aghq}.
+#' @param x The return value of \code{aghq::aghq}. Plots are created for the marginal
+#' pdf and cdf of \code{x$transformation$fromtheta(theta)}.
 #' @param ... not used.
 #'
 #' @return Silently plots.
@@ -407,15 +435,15 @@ plot.aghq <- function(x,...) {
   d <- length(x$marginals)
   # Compute pdf and cdf
   pdfandcdf <- vector(mode = 'list',length = d)
-  for (j in 1:d) pdfandcdf[[j]] <- compute_pdf_and_cdf(x$marginals[[j]],interpolation = x$control$interpolation)
+  for (j in 1:d) pdfandcdf[[j]] <- compute_pdf_and_cdf(x$marginals[[j]],interpolation = x$control$interpolation,transformation = x$transformation)
 
   for (j in 1:d) {
-    graphics::plot(pdfandcdf[[j]]$pdf ~ pdfandcdf[[j]]$theta,
+    graphics::plot(pdfandcdf[[j]]$pdf ~ pdfandcdf[[j]]$transparam,
          type = 'l',
          main = paste0("Marg. Post., theta",j),
          xlab = paste0("theta",j),
          ylab = "Density")
-    graphics::plot(pdfandcdf[[j]]$cdf ~ pdfandcdf[[j]]$theta,
+    graphics::plot(pdfandcdf[[j]]$cdf ~ pdfandcdf[[j]]$transparam,
          type = 'l',
          main = paste0("Marg. CDF, theta",j),
          xlab = paste0("theta",j),
@@ -701,6 +729,11 @@ print.laplacesummary <- function(x,...) {
 #' vectors to start the optimizations for each variable. If you're using this method
 #' then the log-joint posterior should be concave and these optimizations should not be
 #' sensitive to starting values.
+#' @param transformation Optional. Do the quadrature for parameter \code{theta}, but
+#' return summaries and plots for parameter \code{g(theta)}. This applies to the \code{theta}
+#' parameters only, not the \code{W} parameters.
+#' \code{transformation} is either: a) an \code{aghqtrans} object returned by \code{aghq::make_transformation},
+#' or b) a list that will be passed to that function internally. See \code{?aghq::make_transformation} for details.
 #' @param control A list with elements
 #' \itemize{
 #' \item{\code{method}: }{optimization method to use for the \code{theta} optimization:
@@ -767,9 +800,11 @@ print.laplacesummary <- function(x,...) {
 # themarginallaplace <- aghq::marginal_laplace(funlist2dmarg,3,list(W = 0,theta = 0))
 #' @export
 #'
-marginal_laplace <- function(ff,k,startingvalue,optresults = NULL,control = default_control_marglaplace(),...) {
+marginal_laplace <- function(ff,k,startingvalue,transformation = default_transformation(),optresults = NULL,control = default_control_marglaplace(),...) {
 
   validate_control(control,type = 'marglaplace')
+  validate_transformation(transformation)
+  transformation <- make_transformation(transformation)
 
   # Negate it if asked
   if (control$negate) {
@@ -973,7 +1008,8 @@ marginal_laplace <- function(ff,k,startingvalue,optresults = NULL,control = defa
     marginals = marginals,
     optresults = outeropt,
     modesandhessians = modesandhessians,
-    control = control
+    control = control,
+    transformation = transformation
   )
   class(out) <- c("marginallaplace","aghq")
   out
@@ -1003,6 +1039,11 @@ marginal_laplace <- function(ff,k,startingvalue,optresults = NULL,control = defa
 #' **negated** log-posterior and its derivatives. This is **opposite**
 #' to every other comparable function in the \code{aghq} package, and is done
 #' here to emphasize compatibility with \code{TMB}.
+#' @param transformation Optional. Do the quadrature for parameter \code{theta}, but
+#' return summaries and plots for parameter \code{g(theta)}. This applies to the \code{theta}
+#' parameters only, not the \code{W} parameters.
+#' \code{transformation} is either: a) an \code{aghqtrans} object returned by \code{aghq::make_transformation},
+#' or b) a list that will be passed to that function internally. See \code{?aghq::make_transformation} for details.
 #' @param control A list of control parameters. See \code{?default_control} for details. Valid options are:
 #' \itemize{
 #' \item{\code{method}: }{optimization method to use for the \code{theta} optimization:
@@ -1031,13 +1072,15 @@ marginal_laplace <- function(ff,k,startingvalue,optresults = NULL,control = defa
 #'
 #' @export
 #'
-marginal_laplace_tmb <- function(ff,k,startingvalue,optresults = NULL,basegrid = NULL,control = default_control_tmb(),...) {
+marginal_laplace_tmb <- function(ff,k,startingvalue,transformation = default_transformation(),optresults = NULL,basegrid = NULL,control = default_control_tmb(),...) {
 
   validate_control(control,type='tmb')
+  validate_transformation(transformation)
+  transformation <- make_transformation(transformation)
 
   # Get names from TMB function template
   thetanames <- NULL
-  if (exists('par',ff)) thetanames <- names(ff$par)
+  if (exists('par',ff)) thetanames <- make.unique(names(ff$par),sep='')
 
   # Hessian
   if (control$numhessian) {
@@ -1045,7 +1088,7 @@ marginal_laplace_tmb <- function(ff,k,startingvalue,optresults = NULL,basegrid =
   }
   ## Do aghq ##
   # The aghq
-  quad <- aghq(ff = ff,k = k,startingvalue = startingvalue,optresults = optresults,basegrid = basegrid,control = control,...)
+  quad <- aghq(ff = ff,k = k,transformation = transformation,startingvalue = startingvalue,optresults = optresults,basegrid = basegrid,control = control,...)
   if (control$onlynormconst) return(quad) # NOTE: control was passed to aghq here so quad should itself just be a number
 
   ## Add on the info needed for marginal Laplace ##
@@ -1106,8 +1149,7 @@ marginal_laplace_tmb <- function(ff,k,startingvalue,optresults = NULL,basegrid =
 #' to this effect. Default \code{30}.
 #' @param ... not used.
 #'
-#' @return A list containing an object of class \code{aghqsummary} (see \code{summary.aghq})
-#' plus the following objects:
+#' @return A list containing an object of class \code{aghqsummary} (see \code{summary.aghq}).
 #'
 #' @family quadrature
 #'
